"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const assertValidKey = (key) => {
    if (typeof key != 'string' || key[0] == '/') {
        throw new SyntaxError(`S3 keys must be a string and cannot start with a forward slash (got "${key}")`);
    }
};
class S3 {
    constructor(settings) {
        this.bucket = settings.bucket;
        this.s3 = new aws_sdk_1.default.S3({
            accessKeyId: settings.accessKeyID,
            secretAccessKey: settings.secretAccessKey,
            region: settings.region,
        });
    }
    copy({ fromKey, toKey, contentType, metadata, }) {
        return __awaiter(this, void 0, void 0, function* () {
            assertValidKey(fromKey);
            assertValidKey(toKey);
            yield this.s3.copyObject({
                Bucket: this.bucket,
                CopySource: this.bucket + '/' + fromKey,
                Key: toKey,
                ContentType: contentType,
                Metadata: metadata,
                MetadataDirective: 'REPLACE',
            }).promise();
        });
    }
    getMetadata(key) {
        return __awaiter(this, void 0, void 0, function* () {
            assertValidKey(key);
            const data = yield this.s3.headObject({
                Bucket: this.bucket,
                Key: key,
            }).promise();
            return {
                key: key,
                metadata: data.Metadata || {},
            };
        });
    }
    setMetadata({ key, metadata }) {
        return __awaiter(this, void 0, void 0, function* () {
            assertValidKey(key);
            const data = yield this.s3.headObject({
                Bucket: this.bucket,
                Key: key,
            }).promise();
            yield this.copy({
                fromKey: key,
                toKey: key,
                contentType: data.ContentType,
                metadata: metadata,
            });
        });
    }
    list({ prefix, maximumKeys = 100000 }) {
        return __awaiter(this, void 0, void 0, function* () {
            assertValidKey(prefix);
            const data = yield this.s3.listObjectsV2({
                Bucket: this.bucket,
                Prefix: prefix,
                MaxKeys: maximumKeys,
            }).promise();
            return data.Contents.map(o => ({
                key: o.Key,
                lastModified: o.LastModified,
                size: o.Size,
            }));
        });
    }
    delete(...keys) {
        return __awaiter(this, void 0, void 0, function* () {
            keys.forEach(key => assertValidKey(key));
            const data = yield this.s3.deleteObjects({
                Bucket: this.bucket,
                Delete: {
                    Objects: keys.map(k => ({ Key: k })),
                },
            }).promise();
            data.Deleted.forEach(obj => {
                console.log(`Deleted ${obj.Key}`);
            });
            data.Errors.forEach(obj => {
                console.log(`Failed to delete ${obj.Key}: ${obj.Message}`);
            });
            if (data.Errors.length) {
                throw new Error('Some files failed to delete');
            }
        });
    }
    upload({ key, data, contentType, metadata }) {
        return __awaiter(this, void 0, void 0, function* () {
            assertValidKey(key);
            yield this.s3.upload({
                Bucket: this.bucket,
                Key: key,
                Body: data,
                ContentType: contentType,
                Metadata: metadata,
            });
        });
    }
    stream(key) {
        assertValidKey(key);
        return this.s3.getObject({
            Bucket: this.bucket,
            Key: key,
        }).createReadStream();
    }
}
exports.S3 = S3;
